<!DOCTYPE html>
<html>
<head lang="en">
    <meta charset="UTF-8">
    <title>{% block title %}{% endblock %}</title>
    <style>
        table, th, td {
            border: 1px solid black;
        }

        table {
            border-collapse: collapse;
        }

        .ooidc-block {
            border: solid 1px black;
            margin: 5px;
            padding: 3px;
        }

        .ooidc-block-title {
            display: block;
            font-style: italic;
        }
    </style>
</head>
<body>
{% block content %}{% endblock %}
</body>
</html>