{% extends "djangooidc/base.html" %}

{% block title %}Error{% endblock %}

{% block  content %}
    <div>
        <div>OpenID Connect authentication has failed</div>
        <div>
            <div><span>Error message is: </span>{{ error }}</div>
            {% if callback %}
                <div>Query content was:</div>
                <div>
                    <table>
                        <thead>
                        <tr>
                            <th>Key</th>
                            <th>Value</th>
                        </tr>
                        </thead>
                        <tbody>
                        {% for key,value in callback.items %}
                            <tr>
                                <td>{{ key }}</td>
                                <td>{{ value }}</td>
                            </tr>
                        {% endfor %}
                        </tbody>
                    </table>
                </div>
            {% endif %}
        </div>
    </div>
{% endblock %}