{% extends "djangooidc/base.html" %}
{% block title %}Login{% endblock %}
{% block content %}
<div>
    <div>Please log in with one of the following methods:</div>
    <div class="ooidc-block">
        <div class="ooidc-block-title">Application login</div>
        <form method="post">
            {% csrf_token %}
            {{ ilform }}
            <input type="hidden" name="next" value="{{ next }}" />
            <input name="internal_login" type="submit">
        </form>
    </div>

    {% if op_list %}
    <div class="ooidc-block">
        <div class="ooidc-block-title">Log in with one of the following systems</div>
        {% for op_name in op_list %}
        <a href="{% url 'openid_with_op_name' op_name=op_name %}">{{ op_name }}</a>&nbsp;
        {% endfor %}
    </div>
    {% endif %}

    {% if dynamic %}
    <div class="ooidc-block">
        <div class="ooidc-block-title">Log in in with an OpenID Connect provider of your choice</div>
        <form method="post">
            {% csrf_token %}
            {{ form }}
            <input type="submit">
        </form>
    </div>
    {% endif %}
</div>
{% endblock %}